@@ -1,10 +1,7 @@
-use super::code::{self, Register, Convention, Action};
+use std::fmt::{Debug};
+use super::{code, target};
 
-const NUM_REGISTERS: usize = super::target::x86_64::ALLOCATABLE_REGISTERS.len();
-
-fn all_registers() -> impl Iterator<Item=Register> {
-    (0..NUM_REGISTERS).map(|i| Register::new(i as u8).unwrap())
-}
+use code::{Convention, EBB};
 
 //-----------------------------------------------------------------------------
 
@@ -20,41 +17,36 @@ pub use cost::{BUDGET, SPILL_COST, SLOT_COST, Cost, op_cost};
 mod dataflow;
 pub use dataflow::{Dataflow, Node, Out};
 
+mod cft;
+pub use cft::{Switch, Cold, CFT};
+
 mod simulation;
-pub use simulation::{Simulation};
+pub use simulation::{Simulation, simulate};
 
-mod moves;
-pub use moves::{moves};
+mod builder;
+pub use builder::{build};
 
-mod allocator;
-pub use allocator::{Instruction, allocate};
+/** Look up information about a control-flow merge point. */
+pub trait LookupLeaf<L: Clone> {
+    /** Return the convention in effect at `leaf`. */
+    fn after(&self, leaf: &L) -> &Convention;
+    /** Return the estimated relative frequency of `leaf`. */
+    fn weight(&self, leaf: &L) -> usize;
+}
 
-mod codegen;
-pub use codegen::{generate_code};
-
-/** Optimizes a basic block. */
-pub fn optimize(num_globals: usize, before: &Convention, after: &Convention, actions: &[Action]) -> Box<[Action]> {
+/** Optimizes an [`EBB`]. */
+pub fn optimize<L: Clone + Debug>(before: &Convention, input: &EBB<L>, lookup_leaf: &impl LookupLeaf<L>)
+-> EBB<L> {
     // Generate the [`Dataflow`] graph.
-    let mut simulation = Simulation::new(before);
-    for action in actions {
-        simulation.action(action);
-    }
-    let (dataflow, exit_node) = simulation.finish(after);
-    // TODO: enumerate the live [`Node`]s. For now, keep them all.
-    let nodes: Vec<_> = dataflow.all_nodes().collect();
-    assert_eq!(dataflow.entry_node(), nodes[0]);
-    let nodes = &nodes[1..];
-    // Refine the schedule and allocate registers.
-    let (instructions, allocation) = allocate(before, &dataflow, nodes);
-    // Generate the [`Action`]s.
-    generate_code(num_globals, before, after, &dataflow, &instructions, allocation, exit_node)
+    let (dataflow, cft) = simulate(before, input, lookup_leaf);
+    build(before, &dataflow, &cft, lookup_leaf)
 }
 
 //-----------------------------------------------------------------------------
 
 #[cfg(test)]
 mod tests {
-    use std::collections::{HashMap};
+/*
     use super::*;
     use code::{Register, REGISTERS, Slot, UnaryOp, BinaryOp, AliasMask, Precision, Width};
     use code::tests::{Emulator};
@@ -148,4 +140,5 @@ mod tests {
         let _ = optimize(0, &before, &after, &actions);
         // Just don't panic!
     }
+*/
 }
